package logevent.demo1.original;

import java.io.IOException;

import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reporter;

public class LogEventParseMapper extends MapReduceBase implements
		Mapper<LongWritable, Text, Text, IntWritable> {
	
	enum Level { DEBUG, INFO, WARN, ERROR, FATAL };

	@Override
	public void map(LongWritable key, Text value,
			OutputCollector<Text, IntWritable> output, Reporter reporter)
			throws IOException {

		String line = value.toString();
		
		// ignore empty lines
		if (line.trim().isEmpty()) { 
			return;
		}
		
		String[] fields = line.split(" ");
		
		// ensure this line is not malformed
		if (fields.length <= 3) { 
			return;
		}
		
		String levelField = fields[3];
		for (Level level : Level.values()) {
			String levelName = level.name();
			if (levelName.equalsIgnoreCase(levelField)) {
				output.collect(new Text(levelName), new IntWritable(1));
			}
		}
	}
}