package logevent.demo6.failedjob;

import java.io.IOException;
import java.util.Iterator;

import logevent.demo6.failedjob.LogParser.Level;

import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.FileInputFormat;
import org.apache.hadoop.mapred.FileOutputFormat;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;

// The mapper and reducer have been included as inner classes.
//
// NOTE: This code contains an intentional and subtle
// error, used to demonstrate the value of diagnostics.
// So as not to spoil the surprise, the description
// of this is included in a comment at the bottom.
public class LogEventCountJob {
   public static void main(String[] args) throws Exception { 

      JobConf conf = new JobConf(LogEventCountJob.class);
      conf.setJobName("Log Event Counter");

      FileInputFormat.addInputPath(conf, new Path(args[0]));
      FileOutputFormat.setOutputPath(conf, new Path(args[1]));

      conf.setMapperClass(LogEventParseMapper.class);
      conf.setReducerClass(LogEventSumReducer.class);
      conf.setOutputKeyClass(Text.class); 
      conf.setOutputValueClass(IntWritable.class);

      JobClient.runJob(conf);
   }
   
   class LogEventParseMapper extends MapReduceBase implements
	 Mapper<LongWritable, Text, Text, IntWritable> {

		private LogParser parser;
		
		public LogEventParseMapper() {
			parser = new LogParser();
		}
		
		@Override
		public void map(LongWritable key, Text value,
				OutputCollector<Text, IntWritable> output, Reporter reporter)
				throws IOException {
		
			String line = value.toString();
			Level level = parser.parseLine(line);
			
			if (level != null) {
				output.collect(new Text(level.name()), new IntWritable(1));
			}
		}
   }

   class LogEventSumReducer extends MapReduceBase implements
	  Reducer<Text, IntWritable, Text, IntWritable> {

		@Override
		public void reduce(Text key, Iterator<IntWritable> values,
				OutputCollector<Text, IntWritable> output, Reporter reporter)
				throws IOException {
		
			int sum = 0;
			
			while (values.hasNext()) {
				IntWritable value = values.next();
				sum += value.get();
			}
		
			output.collect(key, new IntWritable(sum));
		}
	}
}

// BUG: The mapper and reducer have been brought in as
// inner classes, but since they're not defined with the
// 'static' keyword, Hadoop is unable to instantiate them.