package logevent.demo2.refactored;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import logevent.demo2.refactored.LogParser;
import logevent.demo2.refactored.LogParser.Level;

import org.junit.Before;
import org.junit.Test;

public class LogParserTestCase {
	
	LogParser parser;

	@Before
	public void setUp() throws Exception {
		// do per-test setup here
		parser = new LogParser();
	}
	
	/* Here are the "Happy path" tests */
	
	@Test
	public void testParseLineWorksWithDebug() {
		String line = "2012-09-06 22:16:49.391 CDT DEBUG \"My Message\"";
		assertEquals(Level.DEBUG, parser.parseLine(line));
	}
	
	@Test
	public void testParseLineWorksWithInfo() {
		String line = "2012-09-06 22:16:49.391 CDT INFO \"My Message\"";
		assertEquals(Level.INFO, parser.parseLine(line));
	}
	
	@Test
	public void testParseLineWorksWithWarn() {
		String line = "2012-09-06 22:16:49.391 CDT WARN \"My Message\"";
		assertEquals(Level.WARN, parser.parseLine(line));
	}
	
	@Test
	public void testParseLineWorksWithError() {
		String line = "2012-09-06 22:16:49.391 CDT ERROR \"My Message\"";
		assertEquals(Level.ERROR, parser.parseLine(line));
	}
	
	@Test
	public void testParseLineWorksWithFatal() {
		String line = "2012-09-06 22:16:49.391 CDT FATAL \"My Message\"";
		assertEquals(Level.FATAL, parser.parseLine(line));
	}
	
	/* Here are the corner case tests */
	@Test
	public void testParseLineReturnsNullOnNullLine() {
		assertNull(parser.parseLine(null));
	}
	
	@Test
	public void testParseLineReturnsNullOnEmptyLine() {
		assertNull(parser.parseLine(""));
	}
	
	@Test
	public void testParseLineReturnsNullOnWhitespaceLine() {
		assertNull(parser.parseLine("    "));
	}
	
	@Test
	public void testParseLineReturnsNullOnMalformedLine() {
		assertNull(parser.parseLine("ThisIsNotReallyALogLine"));
	}	
}
